Package: nanotime
Type: Package
Title: Nanosecond-Resolution Time Support for R
Version: 0.3.15
Date: 2026-05-21
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Leonardo", "Silvestri", role = "aut"))
Description: Full 64-bit resolution date and time functionality with
 nanosecond granularity is provided, with easy transition to and from
 the standard 'POSIXct' type. Three additional classes offer interval,
 period and duration functionality for nanosecond-resolution timestamps.
Depends: methods
Imports: bit64 (>= 4.8.0), RcppCCTZ (>= 0.2.9), zoo, Rcpp (>= 1.1.1)
Suggests: tinytest, data.table, xts, ggplot2
LinkingTo: Rcpp, RcppCCTZ, RcppDate
License: GPL (>= 2)
URL: https://github.com/eddelbuettel/nanotime,
        https://eddelbuettel.github.io/nanotime/,
        https://dirk.eddelbuettel.com/code/nanotime.html
BugReports: https://github.com/eddelbuettel/nanotime/issues
RoxygenNote: 7.3.2
Collate: 'nanotime.R' 'nanoival.R' 'nanoduration.R' 'nanoperiod.R'
        'RcppExports.R'
Encoding: UTF-8
NeedsCompilation: yes
VignetteBuilder: Rcpp
Packaged: 2026-05-21 11:52:30 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6419-907X>),
  Leonardo Silvestri [aut]
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2026-05-21 12:30:12 UTC
