# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

duration_from_string_impl <- function(str) {
    .Call(`_nanotime_duration_from_string_impl`, str)
}

duration_to_string_impl <- function(dur) {
    .Call(`_nanotime_duration_to_string_impl`, dur)
}

duration_is_na_impl <- function(dur) {
    .Call(`_nanotime_duration_is_na_impl`, dur)
}

make_duration_impl <- function(h_nv, m_nv, s_nv, n_nv) {
    .Call(`_nanotime_make_duration_impl`, h_nv, m_nv, s_nv, n_nv)
}

nanoduration_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_nanoduration_subset_numeric_impl`, v, idx)
}

nanoduration_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_nanoduration_subset_logical_impl`, v, idx_p)
}

nanoival_intersect_idx_time_interval_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_idx_time_interval_impl`, nv1, nv2)
}

nanoival_intersect_idx_time_interval_logical_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_idx_time_interval_logical_impl`, nv1, nv2)
}

nanoival_intersect_time_interval_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_time_interval_impl`, nv1, nv2)
}

nanoival_setdiff_time_interval_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_setdiff_time_interval_impl`, nv1, nv2)
}

nanoival_union_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_union_impl`, nv1, nv2)
}

nanoival_intersect_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_intersect_impl`, nv1, nv2)
}

nanoival_setdiff_impl <- function(nv1, nv2) {
    .Call(`_nanotime_nanoival_setdiff_impl`, nv1, nv2)
}

nanoival_is_unsorted_impl <- function(nvec, strictlyvec) {
    .Call(`_nanotime_nanoival_is_unsorted_impl`, nvec, strictlyvec)
}

nanoival_sort_impl <- function(nvec, decreasingvec) {
    .Call(`_nanotime_nanoival_sort_impl`, nvec, decreasingvec)
}

nanoival_sort_impl2 <- function(nvec, decreasing) {
    .Call(`_nanotime_nanoival_sort_impl2`, nvec, decreasing)
}

nanoival_lt_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_lt_impl`, n1, n2)
}

nanoival_le_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_le_impl`, n1, n2)
}

nanoival_gt_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_gt_impl`, n1, n2)
}

nanoival_ge_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_ge_impl`, n1, n2)
}

nanoival_eq_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_eq_impl`, n1, n2)
}

nanoival_ne_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_ne_impl`, n1, n2)
}

nanoival_plus_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_plus_impl`, n1, n2)
}

nanoival_minus_impl <- function(n1, n2) {
    .Call(`_nanotime_nanoival_minus_impl`, n1, n2)
}

nanoival_setdiff_idx_time_interval_impl <- function(nv1, cv2) {
    .Call(`_nanotime_nanoival_setdiff_idx_time_interval_impl`, nv1, cv2)
}

nanoival_new_impl <- function(sv, ev, sopenv, eopenv) {
    .Call(`_nanotime_nanoival_new_impl`, sv, ev, sopenv, eopenv)
}

nanoival_get_start_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_start_impl`, cv)
}

nanoival_get_end_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_end_impl`, cv)
}

nanoival_get_sopen_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_sopen_impl`, cv)
}

nanoival_get_eopen_impl <- function(cv) {
    .Call(`_nanotime_nanoival_get_eopen_impl`, cv)
}

nanoival_isna_impl <- function(cv) {
    .Call(`_nanotime_nanoival_isna_impl`, cv)
}

nanoival_make_impl <- function(nt_v, tz_v) {
    .Call(`_nanotime_nanoival_make_impl`, nt_v, tz_v)
}

nanoival_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_nanoival_subset_numeric_impl`, v, idx)
}

nanoival_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_nanoival_subset_logical_impl`, v, idx_p)
}

nanotime_wday_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_wday_impl`, tm_v, tz_v)
}

nanotime_mday_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_mday_impl`, tm_v, tz_v)
}

nanotime_month_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_month_impl`, tm_v, tz_v)
}

nanotime_year_impl <- function(tm_v, tz_v) {
    .Call(`_nanotime_nanotime_year_impl`, tm_v, tz_v)
}

nanotime_make_impl <- function(nt_v, tz_v) {
    .Call(`_nanotime_nanotime_make_impl`, nt_v, tz_v)
}

nanotime_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_nanotime_subset_numeric_impl`, v, idx)
}

nanotime_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_nanotime_subset_logical_impl`, v, idx_p)
}

period_from_string_impl <- function(str) {
    .Call(`_nanotime_period_from_string_impl`, str)
}

period_from_parts_impl <- function(months_v, days_v, dur_v) {
    .Call(`_nanotime_period_from_parts_impl`, months_v, days_v, dur_v)
}

period_to_string_impl <- function(prd) {
    .Call(`_nanotime_period_to_string_impl`, prd)
}

period_from_integer64_impl <- function(i64) {
    .Call(`_nanotime_period_from_integer64_impl`, i64)
}

period_from_integer_impl <- function(iint) {
    .Call(`_nanotime_period_from_integer_impl`, iint)
}

period_from_double_impl <- function(dbl) {
    .Call(`_nanotime_period_from_double_impl`, dbl)
}

plus_period_period_impl <- function(e1_nv, e2_nv) {
    .Call(`_nanotime_plus_period_period_impl`, e1_nv, e2_nv)
}

minus_period_impl <- function(e1_cv) {
    .Call(`_nanotime_minus_period_impl`, e1_cv)
}

minus_period_period_impl <- function(e1_cv, e2_cv) {
    .Call(`_nanotime_minus_period_period_impl`, e1_cv, e2_cv)
}

eq_period_period_impl <- function(e1_p, e2_p) {
    .Call(`_nanotime_eq_period_period_impl`, e1_p, e2_p)
}

ne_period_period_impl <- function(e1_p, e2_p) {
    .Call(`_nanotime_ne_period_period_impl`, e1_p, e2_p)
}

plus_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_plus_period_integer64_impl`, e1_cv, e2_nv)
}

minus_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_minus_period_integer64_impl`, e1_cv, e2_nv)
}

multiplies_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_multiplies_period_integer64_impl`, e1_cv, e2_nv)
}

divides_period_integer64_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_divides_period_integer64_impl`, e1_cv, e2_nv)
}

multiplies_period_double_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_multiplies_period_double_impl`, e1_cv, e2_nv)
}

divides_period_double_impl <- function(e1_cv, e2_nv) {
    .Call(`_nanotime_divides_period_double_impl`, e1_cv, e2_nv)
}

minus_integer64_period_impl <- function(e1_nv, e2_cv) {
    .Call(`_nanotime_minus_integer64_period_impl`, e1_nv, e2_cv)
}

plus_nanotime_period_impl <- function(e1_nv, e2_cv, tz_v) {
    .Call(`_nanotime_plus_nanotime_period_impl`, e1_nv, e2_cv, tz_v)
}

minus_nanotime_period_impl <- function(e1_nv, e2_cv, tz_v) {
    .Call(`_nanotime_minus_nanotime_period_impl`, e1_nv, e2_cv, tz_v)
}

plus_nanoival_period_impl <- function(e1_cv, e2_cv, tz_v) {
    .Call(`_nanotime_plus_nanoival_period_impl`, e1_cv, e2_cv, tz_v)
}

minus_nanoival_period_impl <- function(e1_cv, e2_cv, tz_v) {
    .Call(`_nanotime_minus_nanoival_period_impl`, e1_cv, e2_cv, tz_v)
}

period_month_impl <- function(e_n) {
    .Call(`_nanotime_period_month_impl`, e_n)
}

period_day_impl <- function(e_n) {
    .Call(`_nanotime_period_day_impl`, e_n)
}

period_duration_impl <- function(e_n) {
    .Call(`_nanotime_period_duration_impl`, e_n)
}

period_isna_impl <- function(cv) {
    .Call(`_nanotime_period_isna_impl`, cv)
}

period_seq_from_to_impl <- function(from_nv, to_nv, by_cv, tz) {
    .Call(`_nanotime_period_seq_from_to_impl`, from_nv, to_nv, by_cv, tz)
}

period_seq_from_length_impl <- function(from_nv, by_cv, n_nv, tz) {
    .Call(`_nanotime_period_seq_from_length_impl`, from_nv, by_cv, n_nv, tz)
}

period_subset_numeric_impl <- function(v, idx) {
    .Call(`_nanotime_period_subset_numeric_impl`, v, idx)
}

period_subset_logical_impl <- function(v, idx_p) {
    .Call(`_nanotime_period_subset_logical_impl`, v, idx_p)
}

ceiling_tz_impl <- function(nt_v, prd_v, orig_v, tz_v) {
    .Call(`_nanotime_ceiling_tz_impl`, nt_v, prd_v, orig_v, tz_v)
}

ceiling_impl <- function(nt_v, dur_v, orig_v) {
    .Call(`_nanotime_ceiling_impl`, nt_v, dur_v, orig_v)
}

floor_tz_impl <- function(nt_v, prd_v, orig_v, tz_v) {
    .Call(`_nanotime_floor_tz_impl`, nt_v, prd_v, orig_v, tz_v)
}

floor_impl <- function(nt_v, dur_v, orig_v) {
    .Call(`_nanotime_floor_impl`, nt_v, dur_v, orig_v)
}

